@@ -35,7 +35,7 @@
 #include <gtk/gtk.h>
 
 #include <libxfce4util/libxfce4util.h>
-#include <libxfcegui4/dialogs.h>
+#include <libxfce4ui/libxfce4ui.h>
 #include "cpu.h" 
 
 #if defined(__linux__) || defined(__FreeBSD_kernel__)
@@ -51,10 +51,10 @@ struct cpu_load_struct {
 
 gulong cpu_used, oldtotal, oldused;
 
-gulong read_cpuload()
+gulong read_cpuload(void)
 {
     FILE *fd;
-    uint64_t user, nice, system, idle, iowait, irq, softirq, guest;
+    unsigned long long int user, unice, usystem, idle, iowait, irq, softirq, guest;
     gulong used, total;
     int nb_read;
 
@@ -65,8 +65,8 @@ gulong read_cpuload()
     }
 
     /* Don't count steal time. It is neither busy nor free tiime. */
-    nb_read = fscanf (fd, "%*s " "%llu %llu %llu %llu %llu %llu %llu %*llu %llu",
-	    &user, &nice, &system, &idle, &iowait, &irq, &softirq, &guest);
+    nb_read = fscanf (fd, "%*s %llu %llu %llu %llu %llu %llu %llu %*u %llu",
+	    &user, &unice, &usystem, &idle, &iowait, &irq, &softirq, &guest);
     fclose(fd);
     switch (nb_read) /* fall through intentional */
     {
@@ -80,7 +80,7 @@ gulong read_cpuload()
 		    guest = 0;
     }
 
-    used = user + nice + system + irq + softirq + guest;
+    used = user + unice + usystem + irq + softirq + guest;
     total = used + idle + iowait;
 
     if ((total - oldtotal) != 0)
@@ -118,7 +118,7 @@ struct cpu_load_struct {
 
 gulong cpu_used, oldtotal, oldused;
 
-gulong read_cpuload()
+gulong read_cpuload(void)
 {
     gulong used, total;
     long cp_time[CPUSTATES];
@@ -170,7 +170,7 @@ struct cpu_load_struct {
 
 gulong cpu_used, oldtotal, oldused;
 
-gulong read_cpuload()
+gulong read_cpuload(void)
 {
     gulong used, total;
     static int mib[] = { CTL_KERN, KERN_CP_TIME };
@@ -224,7 +224,7 @@ struct cpu_load_struct {
 
 gulong cpu_used, oldtotal, oldused;
 
-gulong read_cpuload()
+gulong read_cpuload(void)
 {
     gulong used, total;
     static int mib[] = { CTL_KERN, KERN_CPTIME };
@@ -252,6 +252,62 @@ gulong read_cpuload()
 
     return cpu_used;
 }
+#elif defined(__sun__)
+
+#include <kstat.h>
+kstat_ctl_t *kc;
+gulong cpu_used;
+gulong oldtotal = 0;
+gulong oldused = 0;
+
+void init_stats()
+{
+       kc = kstat_open();
+}
+
+gulong read_cpuload()
+{
+    gulong used, total;
+    kstat_t *ksp;
+    kstat_named_t *knp;
+
+    if (!kc)
+    {
+       init_stats();
+    }
+    kstat_chain_update(kc);
+    used = 0;
+    total = 0;
+    for (ksp = kc->kc_chain; ksp != NULL; ksp = ksp->ks_next)
+    {
+        if (!strcmp(ksp->ks_module, "cpu") && !strcmp(ksp->ks_name, "sys"))
+       {
+           kstat_read(kc, ksp, NULL);
+           knp = kstat_data_lookup(ksp, "cpu_ticks_user");
+           used += knp->value.ui64;
+           total += knp->value.ui64;
+           knp = kstat_data_lookup(ksp, "cpu_ticks_kernel");
+           used += knp->value.ui64;
+           total += knp->value.ui64;
+           knp = kstat_data_lookup(ksp, "cpu_ticks_idle");
+           total += knp->value.ui64;
+       }
+    }
+
+    printf("CPU: %d %d %d %d\n", used, oldused, total, oldtotal);
+
+    if ((total - oldtotal) != 0)
+    {
+        cpu_used = (100 * (double)(used - oldused)) / (double)(total - oldtotal);
+    }
+    else
+    {
+        cpu_used = 0;
+    }
+    oldused = used;
+    oldtotal = total;
+    return cpu_used;
+}
 
 #else
 #error "Your platform is not yet supported"
