@@ -58,6 +58,13 @@ static gchar *DEFAULT_COLOR[] = { "#0000c0", "#00c000", "#f0f000" };
 
 #define BORDER 8
 
+/* check for new Xfce 4.10 panel features */
+#ifdef LIBXFCE4PANEL_CHECK_VERSION
+#if LIBXFCE4PANEL_CHECK_VERSION (4,9,0)
+#define HAS_PANEL_49
+#endif
+#endif
+
 enum { CPU_MONITOR, MEM_MONITOR, SWAP_MONITOR };
 
 typedef struct
@@ -74,7 +81,6 @@ typedef struct
     GtkWidget  *label;
     GtkWidget  *status;
     GtkWidget  *ebox;
-    GtkWidget  *tooltip_text;
 
     gulong     history[4];
     gulong     value_read;
@@ -86,7 +92,6 @@ typedef struct
 {
     GtkWidget  *label;
     GtkWidget  *ebox;
-    GtkWidget  *tooltip_text;
 
     gulong     value_read;
     gboolean   enabled;
@@ -153,14 +158,14 @@ update_monitors(t_global_monitor *global)
     {
         g_snprintf(caption, sizeof(caption), _("System Load: %ld%%"),
                    global->monitor[0]->value_read);
-        gtk_label_set_text(GTK_LABEL(global->monitor[0]->tooltip_text), caption);
+        gtk_widget_set_tooltip_text(GTK_WIDGET(global->monitor[0]->ebox), caption);
     }
 
     if (global->monitor[1]->options.enabled)
     {
         g_snprintf(caption, sizeof(caption), _("Memory: %ldMB of %ldMB used"),
                    MUsed >> 10 , MTotal >> 10);
-        gtk_label_set_text(GTK_LABEL(global->monitor[1]->tooltip_text), caption);
+        gtk_widget_set_tooltip_text(GTK_WIDGET(global->monitor[1]->ebox), caption);
     }
 
     if (global->monitor[2]->options.enabled)
@@ -171,7 +176,7 @@ update_monitors(t_global_monitor *global)
         else
             g_snprintf(caption, sizeof(caption), _("No swap"));
 
-        gtk_label_set_text(GTK_LABEL(global->monitor[2]->tooltip_text), caption);
+        gtk_widget_set_tooltip_text(GTK_WIDGET(global->monitor[2]->ebox), caption);
     }
 
     if (global->uptime->enabled)
@@ -194,56 +199,47 @@ update_monitors(t_global_monitor *global)
                                caption);
             g_snprintf(caption, sizeof(caption), _("Uptime: %d:%02d"), hours, mins);
         }
-        gtk_label_set_text(GTK_LABEL(global->uptime->tooltip_text), caption);
+        gtk_widget_set_tooltip_text(GTK_WIDGET(global->uptime->ebox), caption);
     }
     return TRUE;
 }
 
-static gboolean tooltip_cb0(GtkWidget *widget, gint x, gint y, gboolean keyboard, GtkTooltip *tooltip, t_global_monitor *global)
+static void
+monitor_update_orientation (XfcePanelPlugin  *plugin,
+                            GtkOrientation    panel_orientation,
+                            GtkOrientation    orientation,
+                            t_global_monitor *global)
 {
-        gtk_tooltip_set_custom(tooltip, global->monitor[0]->tooltip_text);
-        return TRUE;
-}
-
-static gboolean tooltip_cb1(GtkWidget *widget, gint x, gint y, gboolean keyboard, GtkTooltip *tooltip, t_global_monitor *global)
-{
-        gtk_tooltip_set_custom(tooltip, global->monitor[1]->tooltip_text);
-        return TRUE;
-}
-
-static gboolean tooltip_cb2(GtkWidget *widget, gint x, gint y, gboolean keyboard, GtkTooltip *tooltip, t_global_monitor *global)
-{
-        gtk_tooltip_set_custom(tooltip, global->monitor[2]->tooltip_text);
-        return TRUE;
-}
-
-static gboolean tooltip_cb3(GtkWidget *widget, gint x, gint y, gboolean keyboard, GtkTooltip *tooltip, t_global_monitor *global)
-{
-        gtk_tooltip_set_custom(tooltip, global->uptime->tooltip_text);
-        return TRUE;
+    gint count;
+    xfce_hvbox_set_orientation(XFCE_HVBOX(global->box), panel_orientation);
+    for(count = 0; count < 3; count++)
+    {
+        xfce_hvbox_set_orientation(XFCE_HVBOX(global->monitor[count]->box), panel_orientation);
+#ifdef HAS_PANEL_49
+        gtk_label_set_angle(GTK_LABEL(global->monitor[count]->label),
+                            (orientation == GTK_ORIENTATION_HORIZONTAL) ? 0 : -90);
+#endif
+        if (panel_orientation == GTK_ORIENTATION_HORIZONTAL)
+        {
+            gtk_progress_bar_set_orientation(GTK_PROGRESS_BAR(global->monitor[count]->status), GTK_PROGRESS_BOTTOM_TO_TOP);
+        }
+        else
+        {
+            gtk_progress_bar_set_orientation(GTK_PROGRESS_BAR(global->monitor[count]->status), GTK_PROGRESS_LEFT_TO_RIGHT);
+        }
+    }
+#ifdef HAS_PANEL_49
+    gtk_label_set_angle(GTK_LABEL(global->uptime->label),
+                        (orientation == GTK_ORIENTATION_HORIZONTAL) ? 0 : -90);
+#endif
 }
 
 static void
-monitor_set_orientation (XfcePanelPlugin *plugin, GtkOrientation orientation,
-                         t_global_monitor *global)
+create_monitor (t_global_monitor *global)
 {
-    gint count, size;
-    size = xfce_panel_plugin_get_size(plugin);
+    gint count;
 
-    gtk_widget_hide(GTK_WIDGET(global->ebox));
-
-    if (global->box)
-        gtk_container_remove(GTK_CONTAINER(global->ebox), 
-                             GTK_WIDGET(global->box));
-    
-    if (orientation == GTK_ORIENTATION_HORIZONTAL)
-    {
-        global->box = gtk_hbox_new(FALSE, 0);
-    }
-    else
-    {
-        global->box = gtk_vbox_new(FALSE, 0);
-    }
+    global->box = xfce_hvbox_new(xfce_panel_plugin_get_orientation(global->plugin), FALSE, 0);
     gtk_widget_show(global->box);
 
     for(count = 0; count < 3; count++)
@@ -253,38 +249,19 @@ monitor_set_orientation (XfcePanelPlugin *plugin, GtkOrientation orientation,
 
         global->monitor[count]->status = GTK_WIDGET(gtk_progress_bar_new());
 
-        if (orientation == GTK_ORIENTATION_HORIZONTAL)
-        {
-            global->monitor[count]->box = GTK_WIDGET(gtk_hbox_new(FALSE, 0));
-            gtk_progress_bar_set_orientation(GTK_PROGRESS_BAR(global->monitor[count]->status), GTK_PROGRESS_BOTTOM_TO_TOP);
-            gtk_widget_set_size_request(GTK_WIDGET(global->monitor[count]->status), BORDER, size - BORDER);
-        }
-        else
-        {
-            global->monitor[count]->box = GTK_WIDGET(gtk_vbox_new(FALSE, 0));
-            gtk_progress_bar_set_orientation(GTK_PROGRESS_BAR(global->monitor[count]->status), GTK_PROGRESS_LEFT_TO_RIGHT);
-            gtk_widget_set_size_request(GTK_WIDGET(global->monitor[count]->status), size - BORDER, BORDER);
-        }
+        global->monitor[count]->box = xfce_hvbox_new(xfce_panel_plugin_get_orientation(global->plugin), FALSE, 0);
 
         gtk_box_pack_start(GTK_BOX(global->monitor[count]->box),
                            GTK_WIDGET(global->monitor[count]->label),
-                           FALSE, FALSE, 0);
+                           FALSE, FALSE, 2);
 
         global->monitor[count]->ebox = gtk_event_box_new();
         gtk_widget_show(global->monitor[count]->ebox);
         gtk_container_add(GTK_CONTAINER(global->monitor[count]->ebox),
                           GTK_WIDGET(global->monitor[count]->box));
 
-        gtk_widget_modify_bg(GTK_WIDGET(global->monitor[count]->status),
-                             GTK_STATE_PRELIGHT,
-                             &global->monitor[count]->options.color);
-        gtk_widget_modify_bg(GTK_WIDGET(global->monitor[count]->status),
-                             GTK_STATE_SELECTED,
-                             &global->monitor[count]->options.color);
-        gtk_widget_modify_base(GTK_WIDGET(global->monitor[count]->status),
-                               GTK_STATE_SELECTED,
-                               &global->monitor[count]->options.color);
         gtk_event_box_set_visible_window(GTK_EVENT_BOX(global->monitor[count]->ebox), FALSE);
+        gtk_event_box_set_above_child(GTK_EVENT_BOX(global->monitor[count]->ebox), TRUE);
 
         gtk_widget_show(GTK_WIDGET(global->monitor[count]->status));
 
@@ -295,15 +272,7 @@ monitor_set_orientation (XfcePanelPlugin *plugin, GtkOrientation orientation,
                            GTK_WIDGET(global->monitor[count]->ebox),
                            FALSE, FALSE, 0);
 
-        if(global->monitor[count]->options.enabled)
-        {
-            gtk_widget_show(GTK_WIDGET(global->monitor[count]->ebox));
-            gtk_widget_show(GTK_WIDGET(global->monitor[count]->box));
-            if (global->monitor[count]->options.use_label)
-                gtk_widget_show(global->monitor[count]->label);
-
-            gtk_widget_show(GTK_WIDGET(global->monitor[count]->status));
-        }
+        gtk_widget_show_all(GTK_WIDGET(global->monitor[count]->ebox));
     }
 
     global->uptime->ebox = gtk_event_box_new();
@@ -311,17 +280,8 @@ monitor_set_orientation (XfcePanelPlugin *plugin, GtkOrientation orientation,
         gtk_widget_show(global->uptime->ebox);
     gtk_event_box_set_visible_window(GTK_EVENT_BOX(global->uptime->ebox), FALSE);
 
-    gtk_widget_set_has_tooltip(global->monitor[0]->ebox, TRUE);
-    gtk_widget_set_has_tooltip(global->monitor[1]->ebox, TRUE);
-    gtk_widget_set_has_tooltip(global->monitor[2]->ebox, TRUE);
-    gtk_widget_set_has_tooltip(global->uptime->ebox, TRUE);
-    g_signal_connect(global->monitor[0]->ebox, "query-tooltip", G_CALLBACK(tooltip_cb0), global);
-    g_signal_connect(global->monitor[1]->ebox, "query-tooltip", G_CALLBACK(tooltip_cb1), global);
-    g_signal_connect(global->monitor[2]->ebox, "query-tooltip", G_CALLBACK(tooltip_cb2), global);
-    g_signal_connect(global->uptime->ebox, "query-tooltip", G_CALLBACK(tooltip_cb3), global);
-
-
     global->uptime->label = gtk_label_new("");
+
     gtk_widget_show(global->uptime->label);
     gtk_container_add(GTK_CONTAINER(global->uptime->ebox),
                       GTK_WIDGET(global->uptime->label));
@@ -331,6 +291,7 @@ monitor_set_orientation (XfcePanelPlugin *plugin, GtkOrientation orientation,
                        FALSE, FALSE, 0);
 
     gtk_container_add(GTK_CONTAINER(global->ebox), GTK_WIDGET(global->box));
+    gtk_event_box_set_visible_window(GTK_EVENT_BOX(global->ebox), FALSE);
     gtk_widget_show(GTK_WIDGET(global->ebox));
 
     update_monitors (global);
@@ -352,7 +313,6 @@ monitor_control_new(XfcePanelPlugin *plugin)
     global->use_timeout_seconds = TRUE;
     global->timeout_id = 0;
     global->ebox = gtk_event_box_new();
-    gtk_container_set_border_width (GTK_CONTAINER (global->ebox), BORDER/2);
     gtk_widget_show(global->ebox);
     global->box = NULL;
 
@@ -373,15 +333,11 @@ monitor_control_new(XfcePanelPlugin *plugin)
         global->monitor[count]->history[1] = 0;
         global->monitor[count]->history[2] = 0;
         global->monitor[count]->history[3] = 0;
-        global->monitor[count]->tooltip_text = gtk_label_new(NULL);
-        g_object_ref(global->monitor[count]->tooltip_text);
 
     }
     
     global->uptime = g_new(t_uptime_monitor, 1);
     global->uptime->enabled = TRUE;
-    global->uptime->tooltip_text = gtk_label_new(NULL);
-    g_object_ref(global->uptime->tooltip_text);
     
     return global;
 }
@@ -405,13 +361,9 @@ monitor_free(XfcePanelPlugin *plugin, t_global_monitor *global)
     {
         if (global->monitor[count]->options.label_text)
             g_free(global->monitor[count]->options.label_text);
-        gtk_widget_destroy(global->monitor[count]->tooltip_text);
         g_free(global->monitor[count]);
     }
 
-    gtk_widget_destroy(global->uptime->tooltip_text);
-
-
     g_free(global->uptime);
 
     g_free(global);
@@ -420,6 +372,7 @@ monitor_free(XfcePanelPlugin *plugin, t_global_monitor *global)
 static void
 setup_timer(t_global_monitor *global)
 {
+    GtkSettings *settings;
     if (global->timeout_id)
         g_source_remove(global->timeout_id);
 #ifdef HAVE_UPOWER_GLIB
@@ -438,6 +391,13 @@ setup_timer(t_global_monitor *global)
     }
 #endif
     global->timeout_id = g_timeout_add(global->timeout, (GSourceFunc)update_monitors, global);
+    /* reduce the default tooltip timeout to be smaller than the update interval otherwise
+     * we won't see tooltips on GTK 2.16 or newer */
+    settings = gtk_settings_get_default();
+    if (g_object_class_find_property(G_OBJECT_GET_CLASS(settings), "gtk-tooltip-timeout"))
+        g_object_set(settings, "gtk-tooltip-timeout",
+                     global->timeout - 10, NULL);
+
 }
 
 static void
@@ -609,18 +569,19 @@ monitor_set_size(XfcePanelPlugin *plugin, int size, t_global_monitor *global)
 {
     gint count;
 
+    gtk_container_set_border_width (GTK_CONTAINER (global->ebox), (size > 26 ? 2 : 1));
     for(count = 0; count < 3; count++)
     {
         if (xfce_panel_plugin_get_orientation (plugin) == 
                 GTK_ORIENTATION_HORIZONTAL)
         {
             gtk_widget_set_size_request(GTK_WIDGET(global->monitor[count]->status),
-                                        BORDER, size - BORDER);
+                                        8, -1);
         }
         else
         {
             gtk_widget_set_size_request(GTK_WIDGET(global->monitor[count]->status),
-                                        size - BORDER, BORDER);
+                                        -1, 8);
         }
     }
     
@@ -629,6 +590,38 @@ monitor_set_size(XfcePanelPlugin *plugin, int size, t_global_monitor *global)
     return TRUE;
 }
 
+
+#ifdef HAS_PANEL_49
+static void
+monitor_set_mode (XfcePanelPlugin *plugin, XfcePanelPluginMode mode,
+                  t_global_monitor *global)
+{
+  GtkOrientation panel_orientation = xfce_panel_plugin_get_orientation (plugin);
+  GtkOrientation orientation = (mode == XFCE_PANEL_PLUGIN_MODE_VERTICAL) ?
+    GTK_ORIENTATION_VERTICAL : GTK_ORIENTATION_HORIZONTAL;
+
+  /* Set small in all modes except deskbar mode */
+  if (mode == XFCE_PANEL_PLUGIN_MODE_DESKBAR)
+      xfce_panel_plugin_set_small(XFCE_PANEL_PLUGIN(plugin), FALSE);
+  else
+      xfce_panel_plugin_set_small(XFCE_PANEL_PLUGIN(plugin), TRUE);
+
+  monitor_update_orientation (plugin, panel_orientation, orientation, global);
+  monitor_set_size (plugin, xfce_panel_plugin_get_size (plugin), global);
+}
+
+
+#else
+static void
+monitor_set_orientation (XfcePanelPlugin *plugin, GtkOrientation orientation,
+                         t_global_monitor *global)
+{
+  monitor_update_orientation (plugin, orientation, GTK_ORIENTATION_HORIZONTAL, global);
+  monitor_set_size (plugin, xfce_panel_plugin_get_size (plugin), global);
+}
+#endif
+
+
 #ifdef HAVE_UPOWER_GLIB
 static void
 upower_changed_cb(UpClient *client, t_global_monitor *global)
@@ -913,9 +906,16 @@ systemload_construct (XfcePanelPlugin *plugin)
 
     monitor_read_config (plugin, global);
     
+    create_monitor (global);
+#ifdef HAS_PANEL_49
+    monitor_set_mode (plugin,
+                      xfce_panel_plugin_get_mode (plugin),
+                      global);
+#else
     monitor_set_orientation (plugin, 
                              xfce_panel_plugin_get_orientation (plugin),
                              global);
+#endif
 
     setup_monitor (global);
 
@@ -938,8 +938,13 @@ systemload_construct (XfcePanelPlugin *plugin)
     g_signal_connect (plugin, "size-changed", G_CALLBACK (monitor_set_size),
                       global);
 
+#ifdef HAS_PANEL_49
+    g_signal_connect (plugin, "mode-changed",
+                      G_CALLBACK (monitor_set_mode), global);
+#else
     g_signal_connect (plugin, "orientation-changed", 
                       G_CALLBACK (monitor_set_orientation), global);
+#endif
 
     xfce_panel_plugin_menu_show_configure (plugin);
     g_signal_connect (plugin, "configure-plugin", 
